@@ -25,6 +25,9 @@
 #include "ignition/gui/MainWindow.hh"
 #include "ignition/gui/Plugin.hh"
 #include "ignition/gui/qt.h"
+#include "ignition/msgs/boolean.pb.h"
+#include "ignition/msgs/server_control.pb.h"
+#include "ignition/transport/Node.hh"
 
 namespace ignition
 {
@@ -48,8 +51,32 @@ namespace ignition
       /// fully initialized.
       public: const unsigned int paintCountMin{20};
 
+      /// \brief The action executed when GUI is closed without prompt.
+      public: ExitAction defaultExitAction{ExitAction::CLOSE_GUI};
+
       /// \brief Show the confirmation dialog on exit
       public: bool showDialogOnExit{false};
+
+      /// \brief Text of the prompt in the confirmation dialog on exit
+      public: QString dialogOnExitText;
+
+      /// \brief Show "shutdown" button in exit dialog
+      public: bool exitDialogShowShutdown{false};
+
+      /// \brief Show "Close GUI" button in exit dialog
+      public: bool exitDialogShowCloseGui{true};
+
+      /// \brief Text of "shutdown" button in exit dialog
+      public: QString exitDialogShutdownText;
+
+      /// \brief Text of "Close GUI" button in exit dialog
+      public: QString exitDialogCloseGuiText;
+
+      /// \brief Service to send server control requests
+      public: std::string controlService{"/server_control"};
+
+      /// \brief Communication node
+      public: ignition::transport::Node node;
     };
   }
 }
@@ -59,7 +86,7 @@ using namespace gui;
 
 /// \brief Strip last component from a path.
 /// \return Original path without its last component.
-/// \ToDo: Move this function to ignition::common::Filesystem
+/// \ToDo: Move this function to common::Filesystem
 std::string dirName(const std::string &_path)
 {
   std::size_t found = _path.find_last_of("/\\");
@@ -70,6 +97,11 @@ std::string dirName(const std::string &_path)
 MainWindow::MainWindow()
   : dataPtr(new MainWindowPrivate)
 {
+  // Expose the ExitAction enum to QML via ExitAction 1.0 module
+  qRegisterMetaType<ExitAction>("ExitAction");
+  qmlRegisterUncreatableMetaObject(gui::staticMetaObject,
+    "ExitAction", 1, 0, "ExitAction", "Error: namespace enum");
+
   // Make MainWindow functions available from all QML files (using root)
   App()->Engine()->rootContext()->setContextProperty("MainWindow", this);
 
@@ -161,6 +193,45 @@ void MainWindow::OnSaveConfigAs(const QString &_path)
   this->SaveConfig(localPath.toStdString());
 }
 
+/////////////////////////////////////////////////
+void MainWindow::OnStopServer()
+{
+  std::function<void(const msgs::Boolean &, const bool)> cb =
+    [](const msgs::Boolean &_rep, const bool _result)
+    {
+      if (_rep.data() && _result)
+      {
+        ignmsg << "Simulation server received shutdown request."
+               << std::endl;
+      }
+      else
+      {
+        ignerr << "There was a problem instructing the simulation server to "
+               << "shutdown. It may keep running." << std::endl;
+      }
+    };
+
+  msgs::ServerControl req;
+  req.set_stop(true);
+  const auto success = this->dataPtr->node.Request(
+    this->dataPtr->controlService, req, cb);
+
+  if (success)
+  {
+    ignmsg << "Request to shutdown the simulation server sent. "
+              "Stopping client now." << std::endl;
+  }
+  else
+  {
+    ignerr << "Calling service [" << this->dataPtr->controlService << "] to "
+           << "stop the server failed. Please check that the "
+           << "<server_control_service> of the GUI is configured correctly and "
+           << "that the server is running in the same IGN_PARTITION and with "
+           << "the same configuration of IGN_TRANSPORT_TOPIC_STATISTICS."
+           << std::endl;
+  }
+}
+
 /////////////////////////////////////////////////
 void MainWindow::SaveConfig(const std::string &_path)
 {
@@ -169,7 +240,7 @@ void MainWindow::SaveConfig(const std::string &_path)
   // Create the intermediate directories if needed.
   // We check for errors when we try to open the file.
   auto dirname = dirName(_path);
-  ignition::common::createDirectories(dirname);
+  common::createDirectories(dirname);
 
   // Open the file
   std::ofstream out(_path.c_str(), std::ios::out);
@@ -847,6 +918,19 @@ void MainWindow::SetShowPluginMenu(const bool _showPluginMenu)
   this->ShowPluginMenuChanged();
 }
 
+/////////////////////////////////////////////////
+ExitAction MainWindow::DefaultExitAction() const
+{
+  return this->dataPtr->defaultExitAction;
+}
+
+/////////////////////////////////////////////////
+void MainWindow::SetDefaultExitAction(ExitAction _defaultExitAction)
+{
+  this->dataPtr->defaultExitAction = _defaultExitAction;
+  this->DefaultExitActionChanged();
+}
+
 /////////////////////////////////////////////////
 bool MainWindow::ShowDialogOnExit() const
 {
@@ -859,3 +943,83 @@ void MainWindow::SetShowDialogOnExit(bool _showDialogOnExit)
   this->dataPtr->showDialogOnExit = _showDialogOnExit;
   this->ShowDialogOnExitChanged();
 }
+
+/////////////////////////////////////////////////
+QString MainWindow::DialogOnExitText() const
+{
+  return this->dataPtr->dialogOnExitText;
+}
+
+/////////////////////////////////////////////////
+void MainWindow::SetDialogOnExitText(
+  const QString &_dialogOnExitText)
+{
+  this->dataPtr->dialogOnExitText = _dialogOnExitText;
+  this->DialogOnExitTextChanged();
+}
+
+/////////////////////////////////////////////////
+bool MainWindow::ExitDialogShowShutdown() const
+{
+  return this->dataPtr->exitDialogShowShutdown;
+}
+
+/////////////////////////////////////////////////
+void MainWindow::SetExitDialogShowShutdown(bool _exitDialogShowShutdown)
+{
+  this->dataPtr->exitDialogShowShutdown = _exitDialogShowShutdown;
+  this->ExitDialogShowShutdownChanged();
+}
+
+/////////////////////////////////////////////////
+bool MainWindow::ExitDialogShowCloseGui() const
+{
+  return this->dataPtr->exitDialogShowCloseGui;
+}
+
+/////////////////////////////////////////////////
+void MainWindow::SetExitDialogShowCloseGui(bool _exitDialogShowCloseGui)
+{
+  this->dataPtr->exitDialogShowCloseGui = _exitDialogShowCloseGui;
+  this->ExitDialogShowCloseGuiChanged();
+}
+
+/////////////////////////////////////////////////
+QString MainWindow::ExitDialogShutdownText() const
+{
+  return this->dataPtr->exitDialogShutdownText;
+}
+
+/////////////////////////////////////////////////
+void MainWindow::SetExitDialogShutdownText(
+  const QString &_exitDialogShutdownText)
+{
+  this->dataPtr->exitDialogShutdownText = _exitDialogShutdownText;
+  this->ExitDialogShutdownTextChanged();
+}
+
+/////////////////////////////////////////////////
+QString MainWindow::ExitDialogCloseGuiText() const
+{
+  return this->dataPtr->exitDialogCloseGuiText;
+}
+
+/////////////////////////////////////////////////
+void MainWindow::SetExitDialogCloseGuiText(
+  const QString &_exitDialogCloseGuiText)
+{
+  this->dataPtr->exitDialogCloseGuiText = _exitDialogCloseGuiText;
+  this->ExitDialogCloseGuiTextChanged();
+}
+
+/////////////////////////////////////////////////
+std::string MainWindow::ServerControlService() const
+{
+  return this->dataPtr->controlService;
+}
+
+/////////////////////////////////////////////////
+void MainWindow::SetServerControlService(const std::string &_service)
+{
+  this->dataPtr->controlService = _service;
+}
